@@ -1,5 +1,5 @@
 ## Copyright (C) 1999 Andy Adler <adler@sce.carleton.ca>
-## Copyright (C) 2008 Sren Hauberg <soren@hauberg.org>
+## Copyright (C) 2008 Søren Hauberg <soren@hauberg.org>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -474,7 +474,7 @@ endfunction
 
 ## The 'andy' edge detector that was present in older versions of 'edge'.
 ## The function body has simply been copied from the old implementation.
-##   -- Soren Hauberg, march 11th, 2008
+##   -- Søren Hauberg, march 11th, 2008
 function [imout, thresh] = andy(im, method, thresh, param2)
    [n,m]= size(im);
    xx= 2:m-1;
